/*

  Example of FFI extension I started from:

    https://github.com/pytorch/extension-ffi.git

  There is this tutorial

    https://github.com/pytorch/tutorials/blob/master/Creating%20Extensions%20using%20FFI.md

  And TH's Tensor definition are here in my install:

    anaconda3/lib/python3.5/site-packages/torch/lib/include/TH/generic/THTensor.h

 */

#include <TH/TH.h>

#include "sequence_generator.h"

THByteTensor *generate_sequence(long nb_sequences,
                                long nb_images,
                                long image_height, long image_width,
                                int random_shape_size, int random_colors) {

  long nb_channels = 3;
  unsigned char *a, *b;
  long s, c, k, i, j, st0, st1, st2, st3, st4;

  THLongStorage *size = THLongStorage_newWithSize(5);
  size->data[0] = nb_sequences;
  size->data[1] = nb_images;
  size->data[2] = nb_channels;
  size->data[3] = image_height;
  size->data[4] = image_width;
  THByteTensor *result = THByteTensor_newWithSize(size, NULL);
  THLongStorage_free(size);

  st0 = THByteTensor_stride(result, 0);
  st1 = THByteTensor_stride(result, 1);
  st2 = THByteTensor_stride(result, 2);
  st3 = THByteTensor_stride(result, 3);
  st4 = THByteTensor_stride(result, 4);

  unsigned char tmp_buffer[nb_images * nb_channels * image_width * image_height];

  for(s = 0; s < nb_sequences; s++) {
    a = THByteTensor_storage(result)->data + THByteTensor_storageOffset(result) + s * st0;
    fl_generate_sequence(nb_images, image_width, image_height,
                         random_shape_size, random_colors,
                         tmp_buffer);
    unsigned char *r = tmp_buffer;
    for(k = 0; k < nb_images; k++) {
      for(c = 0; c < nb_channels; c++) {
        for(i = 0; i < image_height; i++) {
          b = a + k * st1 + c * st2 + i * st3;
          for(j = 0; j < image_width; j++) {
            *b = (unsigned char) (*r);
            r++;
            b += st4;
          }
        }
      }
    }
  }

  return result;
}
