
/*
 *  dyncnn is a deep-learning algorithm for the prediction of
 *  interacting object dynamics
 *
 *  Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 *  Written by Francois Fleuret <francois.fleuret@idiap.ch>
 *
 *  This file is part of dyncnn.
 *
 *  dyncnn is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  dyncnn is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with dyncnn.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef UNIVERSE_H
#define UNIVERSE_H

#include <iostream>
#include <cmath>

#include "misc.h"
#include "canvas.h"
#include "polygon.h"

using namespace std;

class Universe {
  scalar_t _width, _height;
public:
  int _nb_max_polygons, _nb_polygons;
  Polygon **_polygons;

  inline scalar_t width() { return _width; }
  inline scalar_t height() { return _height; }

  Universe(int nb_max_polygons, scalar_t width, scalar_t height);
  // The destructor deletes all the added polygons
  ~Universe();

  void initialize_polygon(Polygon *p);
  void clear();
  void add_polygon(Polygon *p);

  bool collide_with_borders(Polygon *p, scalar_t padding);
  bool collide(Polygon *p);

  // Compute collisions between projections of the polygons on a few
  // axis to speed up the computation
  void compute_pseudo_collisions(int nb_axis, int *nb_colliding_axis);

  void apply_collision_forces(scalar_t dt);
  bool update(scalar_t dt, scalar_t padding);

  Polygon *pick_polygon(scalar_t x, scalar_t y);

  void draw(Canvas *canvas);
};

#endif
